@@ -38,19 +38,6 @@
 #include "pocl_device.h"
 #include "config.h"
 
-/*
- * During pocl kernel compiler transformations we use the fixed address 
- * space ids of clang's -ffake-address-space-map to mark the different 
- * address spaces to keep the processing target-independent. These
- * are converted to the target's address space map (if any), in a final
- * kernel compiler pass.
- */
-#define POCL_ADDRESS_SPACE_PRIVATE 0
-#define POCL_ADDRESS_SPACE_GLOBAL 1
-#define POCL_ADDRESS_SPACE_LOCAL 2
-#define POCL_ADDRESS_SPACE_CONSTANT 3
-#define POCL_ADDRESS_SPACE_GENERIC 4
-
 #define POCL_FILENAME_LENGTH 1024
 
 typedef struct _mem_mapping mem_mapping_t;
@@ -62,6 +49,22 @@ struct _mem_mapping {
   mem_mapping_t *prev, *next;
 };
 
+typedef struct _mem_destructor_callback mem_destructor_callback_t;
+/* represents a memory object destructor callback */
+struct _mem_destructor_callback
+{
+  void (CL_CALLBACK * pfn_notify) (cl_mem, void*); /* callback function */
+  void *user_data; /* user supplied data passed to callback function */
+  mem_destructor_callback_t *next;
+};
+
+typedef struct _build_program_callback build_program_callback_t;
+struct _build_program_callback
+{
+    void (CL_CALLBACK * callback_function) (cl_program, void*); /* callback function */
+    void *user_data; /* user supplied data passed to callback function */
+};
+
 // Command Queue datatypes
 
 // clEnqueueNDRangeKernel
@@ -71,10 +74,6 @@ typedef struct
   char *tmp_dir; 
   pocl_workgroup wg;
   cl_kernel kernel;
-  /* A list of argument buffers to free after the command has 
-     been executed. */
-  cl_mem *arg_buffers;
-  unsigned arg_buffer_count;
   size_t local_x;
   size_t local_y;
   size_t local_z;
@@ -124,6 +123,8 @@ typedef struct
   void *dst_ptr;
   size_t dst_offset;
   size_t cb;
+  cl_device_id src_dev;
+  cl_device_id dst_dev;
   cl_mem src_buffer;
   cl_mem dst_buffer;
 } _cl_command_copy;
@@ -141,11 +142,43 @@ typedef struct
   void *device_ptr;
   void *host_ptr;
   size_t origin[3];
+  size_t h_origin[3];
   size_t region[3];
-  size_t rowpitch;
-  size_t slicepitch;
+  size_t h_rowpitch;
+  size_t h_slicepitch;
+  size_t b_rowpitch;
+  size_t b_slicepitch;
   cl_mem buffer;
-} _cl_command_rw_image;
+} _cl_command_read_image;
+
+typedef struct
+{
+  void *device_ptr;
+  const void *host_ptr;
+  size_t origin[3];
+  size_t h_origin[3];
+  size_t region[3];
+  size_t h_rowpitch;
+  size_t h_slicepitch;
+  size_t b_rowpitch;
+  size_t b_slicepitch;
+  cl_mem buffer;
+} _cl_command_write_image;
+
+typedef struct
+{
+  cl_device_id dst_device;
+  cl_mem dst_buffer;
+  cl_device_id src_device;
+  cl_mem src_buffer;
+  size_t dst_origin[3];
+  size_t src_origin[3];
+  size_t region[3];
+  size_t dst_rowpitch;
+  size_t dst_slicepitch;
+  size_t src_rowpitch;
+  size_t src_slicepitch;
+} _cl_command_copy_image;
 
 /* clEnqueueUnMapMemObject */
 typedef struct
@@ -166,21 +199,38 @@ typedef struct
   size_t slicepitch;
   void *fill_pixel;
   size_t pixel_size;
+  cl_mem buffer;
 } _cl_command_fill_image;
 
+/* clEnqueueFillBuffer */
 typedef struct
 {
+  cl_mem buffer;
   void* ptr;
   size_t size, offset;
   void* pattern;
   size_t pattern_size;
 } _cl_command_fill;
 
+/* clEnqueueMarkerWithWaitlist */
 typedef struct
 {
   void *data;
+  int has_wait_list;
 } _cl_command_marker;
 
+/* clEnqueueBarrierWithWaitlist */
+typedef _cl_command_marker _cl_command_barrier;
+
+/* clEnqueueMigrateMemObjects */
+typedef struct
+{
+  void *data;
+  size_t num_mem_objects;
+  cl_mem *mem_objects;
+  cl_device_id *source_devices;
+} _cl_command_migrate;
+
 typedef struct
 {
   void* data;
@@ -221,9 +271,13 @@ typedef union
   _cl_command_copy copy;
   _cl_command_map map;
   _cl_command_fill_image fill_image;
-  _cl_command_rw_image rw_image;
+  _cl_command_read_image read_image;
+  _cl_command_write_image write_image;
+  _cl_command_copy_image copy_image;
   _cl_command_marker marker;
+  _cl_command_barrier barrier;
   _cl_command_unmap unmap;
+  _cl_command_migrate migrate;
   _cl_command_fill memfill;
 
   _cl_command_svm_free svm_free;
@@ -233,21 +287,40 @@ typedef union
 } _cl_command_t;
 
 // one item in the command queue
-typedef struct _cl_command_node_struct
+typedef struct _cl_command_node _cl_command_node;
+struct _cl_command_node
 {
   _cl_command_t command;
   cl_command_type type;
-  struct _cl_command_node_struct *next; // for linked-list storage
+  _cl_command_node * volatile next; // for linked-list storage
+  _cl_command_node * volatile prev;
   cl_event event;
-  const cl_event *event_wait_list;
-  cl_uint num_events_in_wait_list;
+  const cl_event *volatile event_wait_list;
   cl_device_id device;
-} _cl_command_node;
+  volatile cl_int ready;
+};
 
 /* Additional LLVM version macros to simplify ifdefs */
+#if (defined LLVM_3_6)
+# define LLVM_OLDER_THAN_3_7 1
+# define LLVM_OLDER_THAN_3_8 1
+# define LLVM_OLDER_THAN_3_9 1
+# define LLVM_OLDER_THAN_4_0 1
+#endif
 
 #if (defined LLVM_3_7)
 # define LLVM_OLDER_THAN_3_8 1
+# define LLVM_OLDER_THAN_3_9 1
+# define LLVM_OLDER_THAN_4_0 1
+#endif
+
+#if (defined LLVM_3_8)
+# define LLVM_OLDER_THAN_3_9 1
+# define LLVM_OLDER_THAN_4_0 1
+#endif
+
+#if (defined LLVM_3_9)
+# define LLVM_OLDER_THAN_4_0 1
 #endif
 
 #endif /* POCL_H */
