@@ -24,13 +24,14 @@
 
 #include "pocl_cl.h"
 #include "devices.h"
+#include "common.h"
 
 CL_API_ENTRY cl_mem CL_API_CALL
-POname(clCreateBuffer)(cl_context context,
-               cl_mem_flags flags,
-               size_t size,
-               void *host_ptr,
-               cl_int *errcode_ret) CL_API_SUFFIX__VERSION_1_0
+POname(clCreateBuffer)(cl_context   context,
+                       cl_mem_flags flags,
+                       size_t       size,
+                       void         *host_ptr,
+                       cl_int       *errcode_ret) CL_API_SUFFIX__VERSION_1_0
 {
   cl_mem mem = NULL;
   cl_device_id device;
@@ -92,6 +93,7 @@ POname(clCreateBuffer)(cl_context context,
         "host_ptr is not NULL, but flags don't specify {COPY|USE}_HOST_PTR\n");
     }
   
+
   for (i = 0; i < context->num_devices; ++i)
     {
       cl_ulong max_alloc;
@@ -108,51 +110,92 @@ POname(clCreateBuffer)(cl_context context,
   mem->parent = NULL;
   mem->map_count = 0;
   mem->mappings = NULL;
+  mem->destructor_callbacks = NULL;
   mem->type = CL_MEM_OBJECT_BUFFER;
   mem->flags = flags;
   mem->is_image = CL_FALSE;
-  mem->packet_size = 0;
-  mem->max_packets = 0;
+  mem->latest_event = NULL;
+  mem->owning_device = NULL;
+  mem->is_pipe = 0;
+  mem->pipe_packet_size = 0;
+  mem->pipe_max_packets = 0;
 
   /* Store the per device buffer pointers always to a known
      location in the buffer (dev_id), even though the context
      might not contain all the devices. */
   mem->device_ptrs =
-    (pocl_mem_identifier*) malloc(pocl_num_devices * sizeof(pocl_mem_identifier));
+    (pocl_mem_identifier*) calloc(pocl_num_devices,
+                                  sizeof(pocl_mem_identifier));
   if (mem->device_ptrs == NULL)
     {
       errcode = CL_OUT_OF_HOST_MEMORY;
       goto ERROR;
-    }  
-  
-  for (i = 0; i < pocl_num_devices; ++i)
-    {
-      mem->device_ptrs[i].mem_ptr = NULL;
     }
 
-  mem->mem_host_ptr = host_ptr; 
-  mem->size = size;
-  mem->context = context;
-  
+  /* init dev pointer structure to ease inter device pointer sharing
+     in ops->alloc_mem_obj */
   for (i = 0; i < context->num_devices; ++i)
     {
-      if (i > 0)
-        POname(clRetainMemObject) (mem);
-      device = context->devices[i];
-      assert (device->ops->alloc_mem_obj != NULL);
-      if (device->ops->alloc_mem_obj (device, mem) != CL_SUCCESS)
+      int dev_id = context->devices[i]->dev_id;
+
+      mem->device_ptrs[dev_id].global_mem_id =
+        context->devices[i]->global_mem_id;
+      mem->device_ptrs[i].available = 1;
+    }
+
+  mem->size = size;
+  mem->context = context;
+  mem->mem_host_ptr = host_ptr;
+  mem->shared_mem_allocation_owner = NULL;
+
+  /* if there is a "special needs" device (hsa) operating in the host memory 
+     let it alloc memory first for shared memory use */
+  if (context->svm_allocdev)
+    {
+      if (context->svm_allocdev->ops->alloc_mem_obj (context->svm_allocdev, mem, host_ptr) != CL_SUCCESS)
         {
           errcode = CL_MEM_OBJECT_ALLOCATION_FAILURE;
           goto ERROR_CLEAN_MEM_AND_DEVICE;
         }
     }
-  
+
+  for (i = 0; i < context->num_devices; ++i)
+    {
+      /* this is already handled iff available */
+      if (context->svm_allocdev == context->devices[i])
+        continue;
+
+      if (i > 0)
+        POname(clRetainMemObject) (mem);
+      device = context->devices[i];
+      assert (device->ops->alloc_mem_obj != NULL);
+      if (device->ops->alloc_mem_obj (device, mem, host_ptr) != CL_SUCCESS)
+        {
+          errcode = CL_MEM_OBJECT_ALLOCATION_FAILURE;
+          goto ERROR_CLEAN_MEM_AND_DEVICE;
+        }
+    }
+
+  /* Some device driver may already have allocated host accessible memory */
+  if (flags & CL_MEM_ALLOC_HOST_PTR && mem->mem_host_ptr == NULL)
+    {
+      assert(mem->shared_mem_allocation_owner == NULL);
+      mem->mem_host_ptr = pocl_memalign_alloc (MAX_EXTENDED_ALIGNMENT, size);
+      if (mem->mem_host_ptr == NULL)
+        {
+          errcode = CL_OUT_OF_HOST_MEMORY;
+          goto ERROR;
+        }
+    }
+
   POCL_RETAIN_OBJECT(context);
-  
+
+  POCL_MSG_PRINT_INFO ("Created Buffer %p\n", mem);
+
   if (errcode_ret != NULL)
     *errcode_ret = CL_SUCCESS;
   return mem;
-  
+
 ERROR_CLEAN_MEM_AND_DEVICE:
   for (j = 0; j < i; ++j)
     {
