@@ -82,10 +82,12 @@ CL_API_SUFFIX__VERSION_1_2
             supported_image_formats[i].image_channel_data_type ==
             image_format->image_channel_data_type)
           {
+            POCL_MEM_FREE(supported_image_formats);
             goto TYPE_SUPPORTED;
           }
       }
 
+    POCL_MEM_FREE(supported_image_formats);
     POCL_MSG_ERR("Requested image format is not supported\n");
     errcode = CL_IMAGE_FORMAT_NOT_SUPPORTED;
     goto ERROR;
@@ -108,7 +110,12 @@ TYPE_SUPPORTED:
     
     size = image_desc->image_width * image_desc->image_height * elem_size * 
       channels;
-    
+
+    if (image_desc->image_depth > 0)
+      {
+        size *= image_desc->image_depth;
+      }
+
     if (row_pitch == 0)
       {
         row_pitch = image_desc->image_width * elem_size * channels;
