@@ -70,8 +70,8 @@ POname(clCreateProgramWithSource)(cl_context context,
     goto ERROR;
   }
 
+  program->buildprogram_callback = NULL;
   program->source = source;
-  program->compiler_options = NULL;
 
   for (i = 0; i < count; ++i)
     {
@@ -94,18 +94,29 @@ POname(clCreateProgramWithSource)(cl_context context,
 
   *source = '\0';
 
-  program->main_build_log[0] = 0;
   program->context = context;
   program->num_devices = context->num_devices;
   program->devices = context->devices;
-  program->kernels = NULL;
   program->build_status = CL_BUILD_NONE;
-  program->read_locks = NULL;
+  /* we set binary type to NONE here. Based on OCL1.2 spec
+     if program will be compiled using clCompileProgram its binary_type
+     will be set to CL_PROGRAM_BINARY_TYPE_COMPILED_OBJECT.
+     clCompileProgram is currently missing in pocl!
+     if program was created by clLinkProgram which is called
+     with the –createlibrary link option its binary_type will be set to
+     CL_PROGRAM_BINARY_TYPE_LIBRARY.
+     clLinkProgram is currently missing in pocl!
+   */
+  program->binary_type = CL_PROGRAM_BINARY_TYPE_NONE;
 
   if ((program->binary_sizes =
        (size_t*) calloc (program->num_devices, sizeof(size_t))) == NULL ||
       (program->binaries = (unsigned char**)
        calloc (program->num_devices, sizeof(unsigned char*))) == NULL ||
+      (program->pocl_binaries = (unsigned char**)
+       calloc (program->num_devices, sizeof(unsigned char*))) == NULL ||
+      (program->pocl_binary_sizes =
+             (size_t*) calloc (program->num_devices, sizeof(size_t))) == NULL ||
       (program->build_log = (char**)
        calloc (program->num_devices, sizeof(char*))) == NULL ||
       ((program->llvm_irs =
@@ -119,7 +130,6 @@ POname(clCreateProgramWithSource)(cl_context context,
       goto ERROR;
     }
 
-
   POCL_RETAIN_OBJECT(context);
 
   if (errcode_ret != NULL)
@@ -133,6 +143,8 @@ ERROR:
     POCL_MEM_FREE(program->build_log);
     POCL_MEM_FREE(program->binaries);
     POCL_MEM_FREE(program->binary_sizes);
+    POCL_MEM_FREE(program->pocl_binaries);
+    POCL_MEM_FREE(program->pocl_binary_sizes);
     POCL_MEM_FREE(program->source);
   }
   POCL_MEM_FREE(program);
