@@ -46,7 +46,7 @@ POname(clCreateSubDevices)(cl_device_id in_device,
    cl_device_id *new_devs = NULL;
    // number of elements in (copies of) properties, including terminating null
    cl_uint num_props = 0;
-   uint i;
+   cl_uint i;
 
    POCL_GOTO_ERROR_COND((in_device == NULL), CL_INVALID_DEVICE);
    POCL_GOTO_ERROR_COND((properties == NULL), CL_INVALID_VALUE);
@@ -71,7 +71,7 @@ POname(clCreateSubDevices)(cl_device_id in_device,
      // error out if the number of CUs per device is 0 or bigger than the number of
      // CUs of in_device
      POCL_GOTO_ERROR_COND(
-       (properties[1] == 0 || properties[1] > in_device->max_compute_units),
+       (properties[1] == 0 || (cl_uint)properties[1] > in_device->max_compute_units),
        CL_INVALID_VALUE);
      // error out if properties isn't zero-terminated
      POCL_GOTO_ERROR_COND(properties[2] != 0, CL_INVALID_VALUE);
@@ -151,7 +151,8 @@ ERROR:
       if (new_devs[i] == NULL)
         break;
       POCL_RELEASE_OBJECT(new_devs[i], new_refcount);
-      POCL_MEM_FREE(new_devs[i]);
+      if (new_refcount == 0)
+        POCL_MEM_FREE(new_devs[i]);
     }
 
     free(new_devs);
