@@ -1,6 +1,6 @@
 #include "pocl_cl.h"
 #include "pocl_util.h"
-
+#include "pocl_timing.h"
 
 CL_API_ENTRY cl_event CL_API_CALL
 POname(clCreateUserEvent)(cl_context     context ,
@@ -9,7 +9,7 @@ POname(clCreateUserEvent)(cl_context     context ,
   int error;
   cl_event event = NULL;
 
-  error = pocl_create_event (&event, context, NULL, CL_COMMAND_USER);
+  error = pocl_create_event (&event, 0, CL_COMMAND_USER, 0, NULL, context);
 
   if (error != CL_SUCCESS)
     {
@@ -17,7 +17,9 @@ POname(clCreateUserEvent)(cl_context     context ,
     }
   else
     {
+      event->pocl_refcount = 1;
       event->status = CL_SUBMITTED;
+      event->context = context;
     }
 
   if (errcode_ret)
