@@ -67,7 +67,7 @@ POname(clEnqueueSVMMap) (cl_command_queue command_queue,
 
   int errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_SVM_MAP,
                                      event, num_events_in_wait_list,
-                                     event_wait_list);
+                                     event_wait_list, 0, NULL);
 
   if (errcode != CL_SUCCESS)
     {
