@@ -119,6 +119,8 @@ POname(clGetDeviceInfo)(cl_device_id   device,
     POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_uint, device->max_read_image_args);
   case CL_DEVICE_MAX_WRITE_IMAGE_ARGS              :
     POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_uint, device->max_write_image_args);
+  case CL_DEVICE_MAX_READ_WRITE_IMAGE_ARGS         :
+    POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_uint, device->max_read_write_image_args);
   case CL_DEVICE_MAX_MEM_ALLOC_SIZE:
     POCL_RETURN_DEVICE_INFO_WITH_IMPL_CHECK(cl_ulong, device->max_mem_alloc_size);
   case CL_DEVICE_IMAGE2D_MAX_WIDTH                 : 
@@ -170,7 +172,11 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_AVAILABLE                         :
     POCL_RETURN_GETINFO(cl_bool, device->available);
   case CL_DEVICE_COMPILER_AVAILABLE                :
+#ifdef OCS_AVAILABLE
     POCL_RETURN_GETINFO(cl_bool, device->compiler_available);
+#else
+    POCL_RETURN_GETINFO(cl_bool, CL_FALSE);
+#endif
   case CL_DEVICE_LINKER_AVAILABLE                  :
     /* TODO currently we return the same availability as the compiler,
      * since if the compiler is available the linker MUST be available
@@ -192,7 +198,13 @@ POname(clGetDeviceInfo)(cl_device_id   device,
   case CL_DEVICE_PROFILE                           : 
     POCL_RETURN_GETINFO_STR(device->profile);
   case CL_DEVICE_VERSION                           : 
-    POCL_RETURN_GETINFO_STR(device->version);
+    {
+      char res[1000];
+      char *hash = device->ops->build_hash(device);
+      snprintf(res, 1000, "%s HSTR: %s", device->version, hash);
+      free(hash);
+      POCL_RETURN_GETINFO_STR(res);
+    }
   case CL_DEVICE_EXTENSIONS                        : 
     POCL_RETURN_GETINFO_STR(device->extensions);
   case CL_DEVICE_PLATFORM                          :
