@@ -38,6 +38,8 @@ POname(clReleaseCommandQueue)(cl_command_queue command_queue) CL_API_SUFFIX__VER
   POCL_RELEASE_OBJECT(command_queue, new_refcount);
   if (new_refcount == 0)
     {
+      POCL_MSG_PRINT_INFO ("Free Command Queue %p\n", command_queue);
+      POname(clFinish)(command_queue);
       pocl_queue_list_delete(command_queue);
       POCL_MEM_FREE(command_queue);
 
