@@ -23,7 +23,7 @@
 */
 
 #include <string.h>
-   
+
 #ifndef _MSC_VER
 #  include <unistd.h>
 #else
@@ -54,12 +54,15 @@ POname(clReleaseProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
          FIXME: this should not be needed if the kernels
          retain the parent program (and release when the kernel
          is released). */
-      for (k=program->kernels; k!=NULL; k=k->next)
+      for (k = program->kernels; k != NULL; k = k->next)
         {
           k->program = NULL;
         }
 
-      if(program->devices != program->context->devices)
+      if (program->buildprogram_callback)
+        POCL_MEM_FREE(program->buildprogram_callback);
+
+      if (program->devices != program->context->devices)
         POCL_MEM_FREE(program->devices);
 
       POCL_MEM_FREE(program->source);
@@ -77,6 +80,19 @@ POname(clReleaseProgram)(cl_program program) CL_API_SUFFIX__VERSION_1_0
           POCL_MEM_FREE(program->build_log[i]);
       POCL_MEM_FREE(program->build_log);
 
+      if (program->num_kernels)
+        {
+          for (i=0; i < program->num_kernels; i++)
+            {
+              if (program->kernel_names)
+                POCL_MEM_FREE(program->kernel_names[i]);
+              if (program->default_kernels && program->default_kernels[i])
+                POname(clReleaseKernel)(program->default_kernels[i]);
+            }
+          POCL_MEM_FREE(program->kernel_names);
+          POCL_MEM_FREE(program->default_kernels);
+        }
+
       POCL_MEM_FREE(program->build_hash);
       POCL_MEM_FREE(program->llvm_irs);
       POCL_MEM_FREE(program);
