@@ -90,7 +90,15 @@ pocl_ttasim_init_device_ops(struct pocl_device_ops *ops)
   ops->run = pocl_tce_run;
   ops->get_timer_value = pocl_ttasim_get_timer_value;
   ops->init_build = pocl_tce_init_build;
+  ops->flush = pocl_tce_flush;
+  ops->join = pocl_tce_join;
+  ops->submit = pocl_tce_submit;
+  ops->compile_kernel = pocl_tce_compile_kernel;
+  ops->broadcast = pocl_broadcast;
+  ops->notify = pocl_tce_notify;
+  ops->update_event = NULL; //pocl_ttasim_update_event;
   ops->build_hash = pocl_tce_build_hash;
+
 }
 
 
@@ -137,6 +145,11 @@ pocl_ttasim_init_device_infos(struct _cl_device_id* dev)
   dev->extensions = TCE_DEVICE_EXTENSIONS;
   dev->llvm_target_triplet = "tce-tut-llvm";
   dev->has_64bit_long = 1;
+  dev->autolocals_to_args = 1;
+
+  dev->global_as_id = 3;
+  dev->local_as_id = 4;
+  dev->constant_as_id = 5;
 
   SETUP_DEVICE_CL_VERSION(TCE_DEVICE_CL_VERSION_MAJOR, TCE_DEVICE_CL_VERSION_MINOR);
 
@@ -539,3 +552,42 @@ pocl_ttasim_get_timer_value (void *data)
   TTASimDevice *d = (TTASimDevice*)data;
   return d->timeStamp();
 }
+
+void pocl_ttasim_update_event (cl_device_id device, cl_event event, cl_int status)
+{
+  switch (status)
+    {
+    case CL_QUEUED:
+      event->status = status;
+      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
+        event->time_queue = device->ops->get_timer_value(device->data);
+      break;
+    case CL_SUBMITTED:
+      event->status = status;
+      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
+        event->time_submit = device->ops->get_timer_value(device->data);
+      break;
+    case CL_RUNNING:
+      event->status = status;
+      if (event->command_type == CL_COMMAND_NDRANGE_KERNEL)
+        break;
+      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE)
+        event->time_start = device->ops->get_timer_value(device->data);
+      break;
+    case CL_COMPLETE:
+      POCL_MSG_PRINT_INFO("TTA: Command complete, event %d\n", event->id);
+      POCL_LOCK_OBJ (event);
+      if (event->queue->properties & CL_QUEUE_PROFILING_ENABLE &&
+          event->command_type != CL_COMMAND_NDRANGE_KERNEL)
+        event->time_end = device->ops->get_timer_value(device->data);
+      device->ops->broadcast(event);
+      pocl_mem_objs_cleanup (event);
+      event->status = CL_COMPLETE;
+
+      POCL_UNLOCK_OBJ (event);
+      break;
+    default:
+      assert("Invalid event status\n");
+      break;
+    }
+}
