@@ -24,7 +24,11 @@
 #ifndef POCL_IMAGE_UTIL_H
 #define POCL_IMAGE_UTIL_H
 
+#include "pocl_cl.h"
+
+#ifdef __GNUC__
 #pragma GCC visibility push(hidden)
+#endif
 
 extern cl_int 
 pocl_check_image_origin_region (const cl_mem image, 
@@ -37,6 +41,10 @@ pocl_get_image_information (cl_channel_order  ch_order,
                             cl_int*           host_channels,
                             cl_int*           host_elem_size);
 
+extern cl_int
+pocl_check_device_supports_image(const cl_mem image,
+                                 const cl_command_queue command_queue);
+
 extern cl_int
 pocl_write_image(cl_mem               image,
                  cl_device_id         device_id,
@@ -55,7 +63,8 @@ pocl_read_image(cl_mem               image,
                 size_t               host_slice_pitch, 
                 void *               ptr);
 
-
+#ifdef __GNUC__
 #pragma GCC visibility pop
+#endif
                    
 #endif
