@@ -40,6 +40,7 @@ POdeclsym(clCreateImage3D)
 POdeclsym(clCreateImage)
 POdeclsym(clCreateKernel)
 POdeclsym(clCreateKernelsInProgram)
+POdeclsym(clCreatePipe)
 POdeclsym(clCreateProgramWithBinary)
 POdeclsym(clCreateProgramWithSource)
 POdeclsym(clCreateSampler)
@@ -47,6 +48,7 @@ POdeclsym(clCreateSubBuffer)
 POdeclsym(clCreateSubDevices)
 POdeclsym(clCreateUserEvent)
 POdeclsym(clEnqueueBarrier)
+POdeclsym(clEnqueueBarrierWithWaitList)
 POdeclsym(clEnqueueCopyBuffer)
 POdeclsym(clEnqueueCopyBufferRect)
 POdeclsym(clEnqueueCopyBufferToImage) 
@@ -55,6 +57,7 @@ POdeclsym(clEnqueueCopyImageToBuffer)
 POdeclsym(clEnqueueMapBuffer)
 POdeclsym(clEnqueueMapImage)
 POdeclsym(clEnqueueMarker) 
+POdeclsym(clEnqueueMigrateMemObjects)
 POdeclsym(clEnqueueNativeKernel)
 POdeclsym(clEnqueueNDRangeKernel)
 POdeclsym(clEnqueueReadBuffer)
@@ -111,6 +114,7 @@ POdeclsym(clSetKernelArg)
 POdeclsym(clSetMemObjectDestructorCallback)
 POdeclsym(clSetUserEventStatus)
 POdeclsym(clUnloadCompiler)
+POdeclsym(clUnloadPlatformCompiler)
 POdeclsym(clWaitForEvents)
 POdeclsym(clEnqueueSVMFree)
 POdeclsym(clEnqueueSVMMap)
