@@ -29,6 +29,10 @@
 extern "C" {
 #endif
 
+#ifdef __GNUC__
+#pragma GCC visibility push(hidden)
+#endif
+
 /* Returns the cpu name as reported by LLVM. */
 char* get_cpu_name();
 
@@ -63,8 +67,8 @@ int pocl_llvm_get_kernel_metadata
  * at a time or control the options through thread safe methods.
  */
 int pocl_llvm_generate_workgroup_function
-(cl_device_id device, cl_kernel kernel,
- size_t local_x, size_t local_y, size_t local_z
+(char* kernel_cachedir, cl_device_id device,
+ cl_kernel kernel, size_t local_x, size_t local_y, size_t local_z
 );
 
 /**
@@ -96,7 +100,7 @@ unsigned pocl_llvm_get_kernel_count(cl_program program);
  * Returns the number of kernels found in the program (may be greater than
  * 'max_num_krn')
  */
-unsigned pocl_llvm_get_kernel_names( cl_program program, const char **knames, unsigned max_num_krn);
+unsigned pocl_llvm_get_kernel_names( cl_program program, char **knames, unsigned max_num_krn);
 
 /** Compile the kernel in infile from LLVM bitcode to native object file for
  * device, into outfile.
@@ -112,6 +116,9 @@ pocl_update_program_llvm_irs(cl_program program, unsigned device_i,
                              cl_device_id device);
 
 
+#ifdef __GNUC__
+#pragma GCC visibility pop
+#endif
 
 
 #ifdef __cplusplus
