@@ -155,8 +155,8 @@ directfuncs = [
     ({
       ivector_t ilogb_(vector_t);
       ivector_t r = ilogb_(x0);
-      iscalar_t jmin = TYPED_CONST(iscalar_t, SHORT_MIN, INT_MIN, LONG_MIN);
-      iscalar_t jmax = TYPED_CONST(iscalar_t, SHORT_MAX, INT_MAX, LONG_MAX);
+      iscalar_t jmin = TYPED_CONST(iscalar_t, SHRT_MIN, INT_MIN, LONG_MIN);
+      iscalar_t jmax = TYPED_CONST(iscalar_t, SHRT_MAX, INT_MAX, LONG_MAX);
       r = r==jmin ? (ivector_t)INT_MIN : r;
       r = r==jmax ? (ivector_t)INT_MAX : r;
       convert_kvector_t(r);
