@@ -28,6 +28,8 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 
 #include "config.h"
 
+#include "pocl.h"
+
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
 #include "llvm/IR/Constants.h"
 #include "llvm/IR/Instructions.h"
@@ -57,8 +59,13 @@ char ImplicitConditionalBarriers::ID = 0;
 void
 ImplicitConditionalBarriers::getAnalysisUsage(AnalysisUsage &AU) const
 {
+#ifdef LLVM_OLDER_THAN_3_9
   AU.addRequired<PostDominatorTree>();
   AU.addPreserved<PostDominatorTree>();
+#else
+  AU.addRequired<PostDominatorTreeWrapperPass>();
+  AU.addPreserved<PostDominatorTreeWrapperPass>();
+#endif
   AU.addRequired<DominatorTreeWrapperPass>();
   AU.addPreserved<DominatorTreeWrapperPass>();
   AU.addPreserved<VariableUniformityAnalysis>();
@@ -90,8 +97,12 @@ ImplicitConditionalBarriers::runOnFunction(Function &F) {
 
   if (!Workgroup::hasWorkgroupBarriers(F))
     return false;
-  
+
+#ifdef LLVM_OLDER_THAN_3_9
   PDT = &getAnalysis<PostDominatorTree>();
+#else
+  PDT = &getAnalysis<PostDominatorTreeWrapperPass>();
+#endif
 
   typedef std::vector<BasicBlock*> BarrierBlockIndex;
   BarrierBlockIndex conditionalBarriers;
@@ -100,7 +111,11 @@ ImplicitConditionalBarriers::runOnFunction(Function &F) {
     if (!Barrier::hasBarrier(b)) continue;
 
     // Unconditional barrier postdominates the entry node.
+#ifdef LLVM_OLDER_THAN_3_9
     if (PDT->dominates(b, &F.getEntryBlock())) continue;
+#else
+    if (PDT->getPostDomTree().dominates(b, &F.getEntryBlock())) continue;
+#endif
 
 #ifdef DEBUG_COND_BARRIERS
     std::cerr << "### found a conditional barrier";
@@ -128,7 +143,11 @@ ImplicitConditionalBarriers::runOnFunction(Function &F) {
     }
     BasicBlock *pred = firstNonBackedgePredecessor(b);
 
+#ifdef LLVM_OLDER_THAN_3_9
     while (!isa<BarrierBlock>(pred) && PDT->dominates(b, pred)) {
+#else
+    while (!isa<BarrierBlock>(pred) && PDT->getPostDomTree().dominates(b, pred)) {
+#endif
 
 #ifdef DEBUG_COND_BARRIERS
       std::cerr << "### looking at BB " << pred->getName().str() << std::endl;
