@@ -79,7 +79,11 @@ ParallelRegion::GenerateTempNames(llvm::BasicBlock *bb)
           name << ".pocl_temp." << tempCounter;
           ++tempCounter;
           tempName = name.str();
+#ifdef LLVM_OLDER_THAN_4_0
       } while (bb->getParent()->getValueSymbolTable().lookup(tempName) != NULL);
+#else
+      } while (bb->getParent()->getValueSymbolTable()->lookup(tempName) != NULL);
+#endif
       instr->setName(tempName);
     }
 }
@@ -160,8 +164,13 @@ ParallelRegion::remap(ValueToValueMapTy &map)
 
     for (BasicBlock::iterator ii = (*i)->begin(), ee = (*i)->end();
          ii != ee; ++ii)
+#ifdef LLVM_OLDER_THAN_3_9
       RemapInstruction(&*ii, map,
                        RF_IgnoreMissingEntries | RF_NoModuleLevelChanges);
+#else
+      RemapInstruction(&*ii, map,
+                       RF_IgnoreMissingLocals | RF_NoModuleLevelChanges);
+#endif
 
 #ifdef DEBUG_REMAP
     std::cerr << endl << "### block after remap: " << std::endl;
@@ -596,7 +605,7 @@ ParallelRegion::LocalIDZLoad()
 {
   if (LocalIDZLoadInstr != NULL) return LocalIDZLoadInstr;
   IRBuilder<> builder(&*(entryBB()->getFirstInsertionPt()));
-  return LocalIDZLoadInstr = 
+  return LocalIDZLoadInstr =
     builder.CreateLoad
     (entryBB()->getParent()->getParent()->getGlobalVariable(POCL_LOCAL_ID_Z_GLOBAL));
 }
@@ -682,7 +691,7 @@ ParallelRegion::InjectPrintF
   #else
   Constant* const_ptr_8 =
     ConstantExpr::getGetElementPtr
-    (PointerType::getUnqual(Type::getInt8Ty(getGlobalContext())), cast<Constant>(stringArg), const_ptr_8_indices);
+    (PointerType::getUnqual(Type::getInt8Ty(M->getContext())), cast<Constant>(stringArg), const_ptr_8_indices);
   #endif
 
   std::vector<Value*> args;
