@@ -24,26 +24,19 @@
 #ifndef _POCL_PARALLEL_REGION_H
 #define _POCL_PARALLEL_REGION_H
 
-#include "BarrierBlock.h"
+#include <vector>
+#include <sstream>
+
 #include "config.h"
-#if (defined LLVM_3_1 or defined LLVM_3_2)
-#include "llvm/BasicBlock.h"
-#include "llvm/LLVMContext.h"
-#else
+
 #include "llvm/IR/BasicBlock.h"
 #include "llvm/IR/LLVMContext.h"
-#endif
-#if (defined LLVM_3_2 or defined LLVM_3_3 or defined LLVM_3_4)
-#include "llvm/Support/CFG.h"
-#else
 #include "llvm/IR/CFG.h"
-#endif
-
 #include "llvm/Transforms/Utils/ValueMapper.h"
 #include "llvm/ADT/SmallPtrSet.h"
 #include "llvm/ADT/SmallVector.h"
-#include <vector>
-#include <sstream>
+
+#include "BarrierBlock.h"
 
 namespace pocl {
 
@@ -57,7 +50,7 @@ class Kernel;
   // It now exposes too much to the clients and leads to hard
   // to track errors when the API is changed.
   class ParallelRegion : public std::vector<llvm::BasicBlock *> {    
-  public:    
+  public:
     typedef llvm::SmallVector<ParallelRegion *, 8> ParallelRegionVector;
 
     ParallelRegion(int forcedRegionId=-1);
@@ -96,7 +89,7 @@ class Kernel;
     void InjectVariablePrintouts();
 
     void InjectPrintF
-        (llvm::Instruction *before, std::string formatStr, 
+        (llvm::Instruction *before, std::string formatStr,
          std::vector<llvm::Value*>& params);
 
     static ParallelRegion *
