@@ -39,8 +39,12 @@
 #include "Workgroup.h"
 #include "LLVMUtils.h"
 
+#include "pocl_cl.h"
+
 #define DEBUG_TARGET_ADDRESS_SPACES
 
+extern cl_device_id currentPoclDevice;
+
 namespace pocl {
 
 using namespace llvm;
@@ -73,32 +77,37 @@ ConvertedType(llvm::Type *type, std::map<unsigned, unsigned> &addrSpaceMap,
           ConvertedType(type->getArrayElementType(),
                         addrSpaceMap, convertedStructsCache),
           type->getArrayNumElements());
+    // TO CLEAN: Check what is the issue with TCE. If it's a TCE-specific problem,
+    // add a runtime check here for the target device, not compile time!
 #ifndef TCE_AVAILABLE
   } else if (type->isStructTy()) {
+
+    // We need to handle the fields of the structs recursively,
+    // converting their address spaces to the target's and
+    // creating a new struct type in the process.
     if (convertedStructsCache[type])
       return convertedStructsCache[type];
 
-    llvm::StructType* t = dyn_cast<llvm::StructType>(type);
-    llvm::StructType* tn;
-    if (!t->isLiteral()) {
-      std::string s = t->getName().str();
+    llvm::StructType* OrigType = dyn_cast<llvm::StructType>(type);
+    llvm::StructType* NewType;
+    if (!OrigType->isLiteral()) {
+      std::string s = OrigType->getName().str();
       s += "_tas_struct";
-      tn = StructType::create(t->getContext(), s);
-      convertedStructsCache[type] = tn;
+      NewType = StructType::create(OrigType->getContext(), s);
     }
     std::vector<llvm::Type*> newtypes;
-    for (llvm::StructType::element_iterator i = t->element_begin(),
-         e = t->element_end(); i < e; ++i) {
+    for (llvm::StructType::element_iterator i = OrigType->element_begin(),
+         e = OrigType->element_end(); i < e; ++i) {
       newtypes.push_back(ConvertedType(*i, addrSpaceMap, convertedStructsCache));
     }
     ArrayRef<Type*> a(newtypes);
-    if (t->isLiteral()) {
-      tn = StructType::get(t->getContext(), a, t->isPacked());
-      convertedStructsCache[type] = tn;
+    if (OrigType->isLiteral()) {
+      NewType = StructType::get(OrigType->getContext(), a, OrigType->isPacked());
     } else {
-      tn->setBody(a, t->isPacked());
+      NewType->setBody(a, OrigType->isPacked());
     }
-    return tn;
+    convertedStructsCache[type] = NewType;
+    return NewType;
 #endif
   } else {
     return type;
@@ -129,14 +138,15 @@ UpdateAddressSpace(llvm::Value& val, std::map<unsigned, unsigned> &addrSpaceMap,
    \returns true if replacement took place (-> BB iterator needs to restart)
 */
 static bool removeASCI(llvm::Value *v, llvm::Instruction *beforeinst,
-                     std::map<unsigned, unsigned> &addrSpaceMap,
-                     std::map<llvm::Type*, llvm::StructType*> &convertedStructsCache) {
+                       std::map<unsigned, unsigned> &addrSpaceMap,
+                       std::map<llvm::Type*,
+                       llvm::StructType*> &convertedStructsCache) {
   if (isa<ConstantExpr>(v)) {
       ConstantExpr *ce = dyn_cast<ConstantExpr>(v);
       Value *in = ce->getAsInstruction();
       AddrSpaceCastInst *asci = dyn_cast<AddrSpaceCastInst>(in);
       assert(asci);
-      if (asci->getDestTy()->getPointerAddressSpace() == POCL_ADDRESS_SPACE_GENERIC) {
+      if (asci->getDestTy()->getPointerAddressSpace() == POCL_FAKE_AS_GENERIC) {
         asci->insertBefore(beforeinst);
         v->replaceAllUsesWith(in);
         in->takeName(v);
@@ -150,11 +160,21 @@ static bool removeASCI(llvm::Value *v, llvm::Instruction *beforeinst,
       Type* DstTy = as->getDestTy();
       if (isa<PointerType>(SrcTy) && isa<PointerType>(DstTy)) {
         if ((DstTy->getPointerAddressSpace() == SrcTy->getPointerAddressSpace())
-            || (DstTy->getPointerAddressSpace() == POCL_ADDRESS_SPACE_GENERIC))
+            || (DstTy->getPointerAddressSpace() == POCL_FAKE_AS_GENERIC))
           {
-            if (DstTy->getPointerAddressSpace() == POCL_ADDRESS_SPACE_GENERIC)
+            if (DstTy->getPointerAddressSpace() == POCL_FAKE_AS_GENERIC)
               UpdateAddressSpace(*as, addrSpaceMap, convertedStructsCache);
             Value* srcVal = as->getOperand(0);
+            // We cannot just replaceAllUsesWith directly as UpdateAddressSpace
+            // might have changed the struct type to a new *_tas_struct type.
+            // In that case we need to replace also the referred types at least
+            // in case of array accesses. See issue #342 which is using events
+            // and events are context saved for each work-item and the GEPs
+            // that refer to the context array still have the old opencl.event_t
+            // type reference and we have converted the opencl.event_t to
+            // a new opencl.event_t_tas_struct and thus the replaceAllUsesWith
+            // fails with an assertion (if LLVM has assertions enabled) due to
+            // the mismatching type.
             as->replaceAllUsesWith(srcVal);
             as->eraseFromParent();
             return true;
@@ -262,10 +282,10 @@ run(llvm::Module &M,
     unhandledFuncs.push_back(&*functionI);
   }
 
-  for (std::vector<llvm::Function*>::iterator i = unhandledFuncs.begin(), 
+  for (std::vector<llvm::Function*>::iterator i = unhandledFuncs.begin(),
          e = unhandledFuncs.end(); i != e; ++i) {
     llvm::Function &F = **i;
-   
+
     /* Convert the FunctionType. Because there is no mutator API in
        LLVM for this, we need to recreate the whole darn function :( */
     SmallVector<Type *, 8> parameters;
@@ -295,52 +315,60 @@ run(llvm::Module &M,
     SmallVector<ReturnInst *, 1> ri;
 
     if (handle_generic_AS) {
-    /* Remove generic address space casts. Converts types with generic AS to
-     * private AS and then removes redundant AS casting instructions */
-    for (llvm::Function::iterator bbi = F.begin(), bbe = F.end(); bbi != bbe;
-         ++bbi)
-      for (llvm::BasicBlock::iterator ii = bbi->begin(), ie = bbi->end(); ii != ie;
-           ++ii) {
 
-        llvm::Instruction *instr = &*ii;
+      /* Remove generic address space casts. Converts types with generic AS to
+       * private AS and then removes redundant AS casting instructions */
+      for (llvm::Function::iterator bbi = F.begin(), bbe = F.end(); bbi != bbe;
+           ++bbi)
+        for (llvm::BasicBlock::iterator ii = bbi->begin(), ie = bbi->end(); ii != ie;
+             ++ii) {
 
-        if (isa<AddrSpaceCastInst>(instr)) {
-            if (removeASCI(instr, nullptr, addrSpaceMap, convertedStructsCache))
-              { ii = bbi->begin(); continue; }
+          llvm::Instruction *instr = &*ii;
+
+          if (isa<AddrSpaceCastInst>(instr)) {
+            if (removeASCI(instr, nullptr, addrSpaceMap,
+                           convertedStructsCache)) {
+              ii = bbi->begin();
+              continue;
+            }
           }
-        if (isa<StoreInst>(instr)) {
+          if (isa<StoreInst>(instr)) {
             StoreInst *st = dyn_cast<StoreInst>(instr);
             Value *pt = st->getPointerOperand();
             if (Operator::getOpcode(pt) == Instruction::AddrSpaceCast) {
-              if (removeASCI(pt, instr, addrSpaceMap, convertedStructsCache))
-                { ii = bbi->begin(); continue; }
+              if (removeASCI(pt, instr, addrSpaceMap,
+                             convertedStructsCache)) {
+                ii = bbi->begin();
+                continue;
+              }
             } else
-              if (st->getPointerAddressSpace() == POCL_ADDRESS_SPACE_GENERIC)
+              if (st->getPointerAddressSpace() == POCL_FAKE_AS_GENERIC)
                 UpdateAddressSpace(*pt, addrSpaceMap, convertedStructsCache);
         }
-        if (isa<LoadInst>(instr)) {
+          if (isa<LoadInst>(instr)) {
             LoadInst *ld = dyn_cast<LoadInst>(instr);
             Value *pt = ld->getPointerOperand();
             if (Operator::getOpcode(pt) == Instruction::AddrSpaceCast) {
-              if (removeASCI(pt, instr, addrSpaceMap, convertedStructsCache))
-                { ii = bbi->begin(); continue; }
+              if (removeASCI(pt, instr, addrSpaceMap, convertedStructsCache)) {
+                ii = bbi->begin();
+                continue;
+              }
             } else
-              if (ld->getPointerAddressSpace() == POCL_ADDRESS_SPACE_GENERIC)
+              if (ld->getPointerAddressSpace() == POCL_FAKE_AS_GENERIC)
                 UpdateAddressSpace(*pt, addrSpaceMap, convertedStructsCache);
         }
-        if (isa<GetElementPtrInst>(instr)) {
+          if (isa<GetElementPtrInst>(instr)) {
             GetElementPtrInst *gep = dyn_cast<GetElementPtrInst>(instr);
             Value *pt = gep->getPointerOperand();
             if (Operator::getOpcode(pt) == Instruction::AddrSpaceCast) {
-                if (removeASCI(pt, instr, addrSpaceMap, convertedStructsCache))
-                  { ii = bbi->begin(); continue; }
-              } else {
-                if (gep->getPointerAddressSpace() == POCL_ADDRESS_SPACE_GENERIC)
-                  UpdateAddressSpace(*pt, addrSpaceMap, convertedStructsCache);
-              }
+              if (removeASCI(pt, instr, addrSpaceMap, convertedStructsCache))
+              { ii = bbi->begin(); continue; }
+            } else {
+              if (gep->getPointerAddressSpace() == POCL_FAKE_AS_GENERIC)
+                UpdateAddressSpace(*pt, addrSpaceMap, convertedStructsCache);
+            }
+          }
         }
-
-      }
     }
 
     class AddressSpaceReMapper : public ValueMapTypeRemapper {
@@ -395,13 +423,13 @@ run(llvm::Module &M,
           ii = bbi->begin();
           continue;
         }
-        
+
         if (!isa<CallInst>(instr)) continue;
 
         llvm::CallInst *call = dyn_cast<CallInst>(instr);
         llvm::Function *calledF = call->getCalledFunction();
         if (funcReplacements.find(calledF) == funcReplacements.end()) continue;
-         
+
         call->setCalledFunction(funcReplacements[calledF]);
       }
   }
@@ -410,14 +438,20 @@ run(llvm::Module &M,
   /* Delete the old functions. */
   while (funcReplacements.size() > 0) {
 
-    if (Workgroup::isKernelToProcess(*i->first)) {
-      FunctionMapping repl;
-      repl[i->first] = i->second;
-      regenerate_kernel_metadata(M, repl);
+    if (currentPoclDevice->spmd) {
+      // The opencl.kernel metadata needs to be maintained only for
+      // SPMD devices (read: HSA at the time of this writing) because
+      // for non-SPMD machines the original kernel function is not
+      // there anymore, but is replaced with a work-group function.
+      if (Workgroup::isKernelToProcess(*i->first)) {
+        FunctionMapping repl;
+        repl[i->first] = i->second;
+        regenerate_kernel_metadata(M, repl);
+      }
     }
 
     if (i->first->getNumUses() > 0) {
-      for (Value::use_iterator ui = i->first->use_begin(), 
+      for (Value::use_iterator ui = i->first->use_begin(),
              ue = i->first->use_end(); ui != ue; ++ui) {
         User* user = (*ui).getUser();
         user->dump();
@@ -431,6 +465,18 @@ run(llvm::Module &M,
     i = funcReplacements.begin();
   }
 
+  if (!currentPoclDevice->spmd) {
+    // The opencl.kernels metadata is stale for non-SPMD machines
+    // which now have the kernel WI function converted to a WG
+    // function.  Remove it.
+    NamedMDNode *KernelsMD = M.getNamedMetadata("opencl.kernels");
+    if (KernelsMD)
+      M.eraseNamedMetadata(KernelsMD);
+    // Ditto for the old opencl.kernel_wg_size_info.
+    NamedMDNode *SizeInfoMD = M.getNamedMetadata("opencl.kernel_wg_size_info");
+    if (SizeInfoMD)
+      M.eraseNamedMetadata(SizeInfoMD);
+  }
 }
 
 #define POCL_AS_FAKE_GENERIC 0
@@ -447,45 +493,55 @@ TargetAddressSpaces::runOnModule(llvm::Module &M) {
    * First we map the Pocl fake AS numbers higher (above 200),
    * then we map that down to real device AS */
 
+  // I will celebrate on the day when we get rid of TAS ;) --Pekka
+
+  llvm::StringRef arch = currentPoclDevice->llvm_target_triplet;
+
+  /* x86_64 supports flattening the address spaces at the backend, but
+     we still flatten them in pocl due to a couple of reasons.
+
+     At least LLVM 3.5 exposes an issue with pocl's printf or another LLVM pass:
+     After the code emission optimizations there appears a
+     PHI node where the two alternative pointer assignments have different
+     address spaces:
+     %format.addr.2347 =
+     phi i8 addrspace(3)* [ %incdec.ptr58, %if.end56 ],
+     [ %format.addr.1, %while.body45.preheader ]
+
+     This leads to an LLVM crash when it tries to generate a no-op bitcast
+     while it won't be such due to the address space difference (I assume).
+     Workaround this by flattening the address spaces to 0 here also for
+     x86_64 until the real culprit is found.
+
+     Another reason is that LoopVectorizer of LLVM 3.7 crashes when it
+     tries to create a masked store intrinsics with the fake address space
+     ids, so we need to flatten them out before vectorizing.
+
+     Seems starting from LLVM 3.8 this situation has improved. Thus, skipping
+     TAS for now until we have the time to get rid off it completely.
+  */
+
+#ifndef LLVM_OLDER_THAN_3_8
+  if (arch.startswith("x86_64"))
+    return false;
+#endif
+
   std::map<unsigned, unsigned> addrSpaceMapUp;
 
-  addrSpaceMapUp[POCL_ADDRESS_SPACE_GLOBAL] = POCL_AS_FAKE_GLOBAL;
-  addrSpaceMapUp[POCL_ADDRESS_SPACE_LOCAL] = POCL_AS_FAKE_LOCAL;
-  addrSpaceMapUp[POCL_ADDRESS_SPACE_GENERIC] = POCL_AS_FAKE_GENERIC;
-  addrSpaceMapUp[POCL_ADDRESS_SPACE_CONSTANT] = POCL_AS_FAKE_CONSTANT;
+  addrSpaceMapUp[POCL_FAKE_AS_GLOBAL] = POCL_AS_FAKE_GLOBAL;
+  addrSpaceMapUp[POCL_FAKE_AS_LOCAL] = POCL_AS_FAKE_LOCAL;
+  addrSpaceMapUp[POCL_FAKE_AS_GENERIC] = POCL_AS_FAKE_GENERIC;
+  addrSpaceMapUp[POCL_FAKE_AS_CONSTANT] = POCL_AS_FAKE_CONSTANT;
 
   run(M, addrSpaceMapUp, true);
 
   std::map<unsigned, unsigned> addrSpaceMapDown;
 
-  llvm::StringRef arch(M.getTargetTriple());
-
   if (arch.startswith("x86_64")) {
-    /* x86_64 supports flattening the address spaces at the backend, but
-       we still flatten them in pocl due to a couple of reasons.
-
-       At least LLVM 3.5 exposes an issue with pocl's printf or another LLVM pass:
-       After the code emission optimizations there appears a
-       PHI node where the two alternative pointer assignments have different
-       address spaces:
-       %format.addr.2347 =
-          phi i8 addrspace(3)* [ %incdec.ptr58, %if.end56 ],
-                               [ %format.addr.1, %while.body45.preheader ]
-
-       This leads to an LLVM crash when it tries to generate a no-op bitcast
-       while it won't be such due to the address space difference (I assume).
-       Workaround this by flattening the address spaces to 0 here also for
-       x86_64 until the real culprit is found.
-
-       Another reason is that LoopVectorizer of LLVM 3.7 crashes when it
-       tries to create a masked store intrinsics with the fake address space
-       ids, so we need to flatten them out before vectorizing.
-    */
     addrSpaceMapDown[POCL_AS_FAKE_GLOBAL] =
         addrSpaceMapDown[POCL_AS_FAKE_LOCAL] =
         addrSpaceMapDown[POCL_AS_FAKE_GENERIC] =
         addrSpaceMapDown[POCL_AS_FAKE_CONSTANT] = 0;
-
   } else if (arch.startswith("arm")) {
     /* Same thing happens here as with x86_64 above.
      */
