@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2012-2015 Tampere University of Technology.
+    Copyright (c) 2012-2017 Tampere University of Technology.
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -24,7 +24,8 @@
  *
  * Definition of WorkItemAliasAnalysis class.
  *
- * @author Vladimír Guzma 2012
+ * @author Vladimír Guzma 2012,
+ * @author Pekka Jääskeläinen 2015-2017
  */
 
 #include <iostream>
@@ -35,7 +36,9 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 #include "pocl.h"
 
 #include "llvm/Analysis/AliasAnalysis.h"
-#include "llvm/Analysis/TargetLibraryInfo.h"
+#ifndef LLVM_OLDER_THAN_3_7
+# include "llvm/Analysis/TargetLibraryInfo.h"
+#endif
 #include "llvm/Analysis/Passes.h"
 #include "llvm/Pass.h"
 #include "llvm/IR/Metadata.h"
@@ -61,7 +64,7 @@ typedef llvm::AliasResult AliasResult;
 #ifdef LLVM_OLDER_THAN_3_8
 class WorkItemAliasAnalysis : public FunctionPass, public AliasAnalysis {
 public:
-    static char ID; 
+    static char ID;
     WorkItemAliasAnalysis() : FunctionPass(ID) {}
 
     /// getAdjustedAnalysisPointer - This method is used when a pass implements
@@ -88,7 +91,7 @@ public:
         return false;
     }
 #endif
-    
+
     private:
         virtual void getAnalysisUsage(AnalysisUsage &AU) const;
         virtual AliasResult alias(const Location &LocA, const Location &LocB);
@@ -104,12 +107,21 @@ class WorkItemAAResult : public AAResultBase<WorkItemAAResult> {
 public:
     static char ID;
 
+#ifdef LLVM_OLDER_THAN_3_9
     WorkItemAAResult(const TargetLibraryInfo &TLI)
         : AAResultBase(TLI) {}
     WorkItemAAResult(const WorkItemAAResult &Arg)
         : AAResultBase(Arg.TLI) {}
     WorkItemAAResult(WorkItemAAResult &&Arg)
         : AAResultBase(Arg.TLI) {}
+#else
+    WorkItemAAResult(const TargetLibraryInfo &)
+        : AAResultBase() {}
+    WorkItemAAResult(const WorkItemAAResult &)
+        : AAResultBase() {}
+    WorkItemAAResult(WorkItemAAResult &&)
+        : AAResultBase() {}
+#endif
 
     AliasResult alias(const MemoryLocation &LocA, const MemoryLocation &LocB);
 };
@@ -118,8 +130,12 @@ class WorkItemAA {
 public:
     typedef WorkItemAAResult Result;
 
+#ifdef LLVM_OLDER_THAN_4_0
     /// \brief Opaque, unique identifier for this analysis pass.
     static void *ID() { return (void *)&PassID; }
+#else
+    static AnalysisKey *ID() { return &PassID; }
+#endif
 
     WorkItemAAResult run(Function &F, AnalysisManager<Function> *AM);
 
@@ -127,7 +143,11 @@ public:
     static StringRef name() { return "WorkItemAliasAnalysis"; }
 
 private:
+#ifdef LLVM_OLDER_THAN_4_0
     static char PassID;
+#else
+    static AnalysisKey PassID;
+#endif
 };
 
 /// Legacy wrapper pass to provide the (WorkItemAAWrapperPass) object.
@@ -148,15 +168,19 @@ public:
     void getAnalysisUsage(AnalysisUsage &AU) const override;
 };
 
+#ifdef LLVM_OLDER_THAN_4_0
 char WorkItemAA::PassID;
+#else
+AnalysisKey WorkItemAA::PassID;
+#endif
 char WorkItemAAResult::ID = 0;
 void WorkItemAliasAnalysis::anchor() {}
 
 WorkItemAAResult WorkItemAA::run(Function &F, AnalysisManager<Function> *AM) {
-    return WorkItemAAResult( AM->getResult<WorkItemAA>(F) );
+    return WorkItemAAResult(AM->getResult<WorkItemAA>(F));
 }
 
-bool WorkItemAliasAnalysis::runOnFunction(llvm::Function &F) {
+bool WorkItemAliasAnalysis::runOnFunction(llvm::Function &) {
     auto &TLIWP = getAnalysis<TargetLibraryInfoWrapperPass>();
     Result.reset(new WorkItemAAResult(TLIWP.getTLI()));
     return false;
@@ -175,9 +199,11 @@ RegisterAnalysisGroup<AliasAnalysis> Y(X);
 RegisterAnalysisGroup<WorkItemAAResult> Y(X);
 #endif
 
+/*
 FunctionPass *createWorkItemAliasAnalysisPass() {
     return new WorkItemAliasAnalysis();
 }
+*/
 
 extern "C" {                                
     FunctionPass*
