@@ -1,19 +1,19 @@
 // LLVM module pass to create the single function (fully inlined)
 // and parallelized kernel for an OpenCL workgroup.
-// 
+//
 // Copyright (c) 2011 Universidad Rey Juan Carlos
 //               2012-2015 Pekka Jääskeläinen
-// 
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -51,6 +51,16 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 #include "Barrier.h"
 #include "Workgroup.h"
 
+#include "LLVMUtils.h"
+#include <cstdio>
+#include <map>
+#include <iostream>
+
+#include "pocl.h"
+#include "pocl_cl.h"
+#include "config.h"
+
+#include "TargetAddressSpaces.h"
 
 #if _MSC_VER
 #  include "vccompat.hpp"
@@ -60,6 +70,8 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 
 POP_COMPILER_DIAGS
 
+extern cl_device_id currentPoclDevice;
+
 using namespace std;
 using namespace llvm;
 using namespace pocl;
@@ -69,8 +81,6 @@ static void privatizeContext(Module &M, Function *F);
 static void createWorkgroup(Module &M, Function *F);
 static void createWorkgroupFast(Module &M, Function *F);
 
-// extern cl::opt<string> Header;
-
 /* The kernel to process in this kernel compiler launch. */
 cl::opt<string>
 KernelName("kernel",
@@ -81,7 +91,7 @@ KernelName("kernel",
 namespace llvm {
 
   typedef struct _pocl_context PoclContext;
-  
+
   template<bool xcompile> class TypeBuilder<PoclContext, xcompile> {
   public:
     static StructType *get(LLVMContext &Context) {
@@ -92,6 +102,7 @@ namespace llvm {
              TypeBuilder<types::i<64>[3], xcompile>::get(Context),
              TypeBuilder<types::i<64>[3], xcompile>::get(Context),
              TypeBuilder<types::i<64>[3], xcompile>::get(Context),
+             TypeBuilder<types::i<64>[3], xcompile>::get(Context),
              NULL);
         }
       else if (size_t_width == 32)
@@ -101,6 +112,7 @@ namespace llvm {
              TypeBuilder<types::i<32>[3], xcompile>::get(Context),
              TypeBuilder<types::i<32>[3], xcompile>::get(Context),
              TypeBuilder<types::i<32>[3], xcompile>::get(Context),
+             TypeBuilder<types::i<32>[3], xcompile>::get(Context),
              NULL);
         }
       else
@@ -126,18 +138,18 @@ namespace llvm {
       WORK_DIM,
       NUM_GROUPS,
       GROUP_ID,
-      GLOBAL_OFFSET
+      GLOBAL_OFFSET,
+      LOCAL_SIZE
     };
   private:
     static int size_t_width;
-    
-  };  
+  };
 
-  template<bool xcompile>  
+  template<bool xcompile>
   int TypeBuilder<PoclContext, xcompile>::size_t_width = 0;
 
 }  // namespace llvm
-  
+
 char Workgroup::ID = 0;
 static RegisterPass<Workgroup> X("workgroup", "Workgroup creation pass");
 
@@ -145,210 +157,231 @@ static RegisterPass<Workgroup> X("workgroup", "Workgroup creation pass");
 bool
 Workgroup::runOnModule(Module &M)
 {
-
-#ifdef LLVM_OLDER_THAN_3_7
-  if (M.getDataLayout()->getPointerSize(0) == 8)
-    {
-      TypeBuilder<PoclContext, true>::setSizeTWidth(64);
-    }
-  else if (M.getDataLayout()->getPointerSize(0) == 4)
-    {
-      TypeBuilder<PoclContext, true>::setSizeTWidth(32);
-    }
-  else 
-    {
-      assert (false && "Target has an unsupported pointer width.");
-    }
-#else
-  if (M.getDataLayout().getPointerSize(0) == 8)
-    {
-      TypeBuilder<PoclContext, true>::setSizeTWidth(64);
-    }
-  else if (M.getDataLayout().getPointerSize(0) == 4)
-    {
-      TypeBuilder<PoclContext, true>::setSizeTWidth(32);
-    }
-  else 
-    {
-      assert (false && "Target has an unsupported pointer width.");
-    }
-#endif
+  switch (currentPoclDevice->address_bits) {
+  case 64:
+    TypeBuilder<PoclContext, true>::setSizeTWidth(64);
+    break;
+  case 32:
+    TypeBuilder<PoclContext, true>::setSizeTWidth(32);
+    break;
+  default:
+    assert (false && "Target has an unsupported pointer width.");
+    break;
+  }
 
   for (Module::iterator i = M.begin(), e = M.end(); i != e; ++i) {
     if (!i->isDeclaration())
       i->setLinkage(Function::InternalLinkage);
   }
 
+  // store the new and old kernel pairs in order to regenerate
+  // all the metadata that used to point to the unmodified
+  // kernels
+  FunctionMapping kernels;
+
   for (Module::iterator i = M.begin(), e = M.end(); i != e; ++i) {
     if (!isKernelToProcess(*i)) continue;
     Function *L = createLauncher(M, &*i);
-      
+
     L->addFnAttr(Attribute::NoInline);
 
     privatizeContext(M, L);
 
-    createWorkgroup(M, L);
-    createWorkgroupFast(M, L);
+    if (!currentPoclDevice->spmd) {
+      createWorkgroup(M, L);
+      createWorkgroupFast(M, L);
+    }
+    else
+      kernels[&*i] = L;
   }
 
-  Function *barrier = cast<Function> 
+  if (currentPoclDevice->spmd) {
+    regenerate_kernel_metadata(M, kernels);
+
+    // Delete the old kernels.
+    for (FunctionMapping::const_iterator i = kernels.begin(),
+           e = kernels.end(); i != e; ++i) {
+        Function *old_kernel = (*i).first;
+        Function *new_kernel = (*i).second;
+        if (old_kernel == new_kernel) continue;
+        old_kernel->eraseFromParent();
+      }
+  }
+
+  Function *barrier = cast<Function>
     (M.getOrInsertFunction(BARRIER_FUNCTION_NAME,
                            Type::getVoidTy(M.getContext()),
                            NULL));
   BasicBlock *bb = BasicBlock::Create(M.getContext(), "", barrier);
   ReturnInst::Create(M.getContext(), 0, bb);
-  
+
   return true;
 }
 
-static Function *
-createLauncher(Module &M, Function *F)
-{
-  SmallVector<Type *, 8> sv;
-
-  for (Function::const_arg_iterator i = F->arg_begin(), e = F->arg_end();
-       i != e; ++i)
-    sv.push_back (i->getType());
-  sv.push_back(TypeBuilder<PoclContext*, true>::get(M.getContext()));
-
-  FunctionType *ft = FunctionType::get(Type::getVoidTy(M.getContext()),
-				       ArrayRef<Type *> (sv),
-				       false);
-
-  std::string funcName = "";
-  funcName = F->getName().str();
-
-  Function *L = Function::Create(ft,
-				 Function::ExternalLinkage,
-				 "_pocl_launcher_" + funcName,
-				 &M);
-
-  SmallVector<Value *, 8> arguments;
-  Function::arg_iterator ai = L->arg_begin();
-  for (unsigned i = 0, e = F->getArgumentList().size(); i != e; ++i)  {
-    arguments.push_back(&*ai);
-    ++ai;
-  }  
-
-  /* Copy the function attributes to transfer noalias etc. from the
-     original kernel which will be inlined into the launcher. */
-  L->setAttributes(F->getAttributes());
+static void addGEPs(llvm::Module &M,
+                    IRBuilder<> &builder,
+                    llvm::Argument *a,
+                    int size_t_width,
+                    int llvmtype,
+                    const char* format_str) {
 
   Value *ptr, *v;
   char s[STRING_LENGTH];
   GlobalVariable *gv;
 
+#ifdef LLVM_OLDER_THAN_3_7
+    ptr = builder.CreateStructGEP(a, llvmtype);
+#else
+    ptr = builder.CreateStructGEP(a->getType()->getPointerElementType(), a, llvmtype );
+#endif
+
+    for (unsigned i = 0; i < 3; ++i) {
+      snprintf(s, STRING_LENGTH, format_str, 'x' + i);
+      gv = M.getGlobalVariable(s);
+      if (gv != NULL) {
+        if (size_t_width == 64) {
+            v = builder.CreateLoad(builder.CreateConstGEP2_64(ptr, 0, i));
+        } else {
+#ifdef LLVM_OLDER_THAN_3_7
+            v = builder.CreateLoad(
+                  builder.CreateConstGEP2_32(ptr, 0, i));
+#else
+            v = builder.CreateLoad(
+              builder.CreateConstGEP2_32(
+                ptr->getType()->getPointerElementType(), ptr, 0, i));
+#endif
+        }
+        builder.CreateStore(v, gv);
+      }
+    }
+}
+
+static Function *
+createLauncher(Module &M, Function *F) {
+
+  SmallVector<Type *, 8> sv;
+
+  for (Function::const_arg_iterator i = F->arg_begin(), e = F->arg_end();
+       i != e; ++i)
+    sv.push_back (i->getType());
+  if (currentPoclDevice->spmd) {
+    PointerType* g_pc_ptr =
+      PointerType::get(TypeBuilder<PoclContext, true>::get(M.getContext()), 1);
+    sv.push_back(g_pc_ptr);
+  } else
+    sv.push_back(TypeBuilder<PoclContext*, true>::get(M.getContext()));
+
+  FunctionType *ft = FunctionType::get(Type::getVoidTy(M.getContext()),
+                                       ArrayRef<Type *> (sv),
+                                       false);
+
+  std::string funcName = "";
+  funcName = F->getName().str();
+  Function *L = NULL;
+  if (currentPoclDevice->spmd) {
+    Function *F = M.getFunction(funcName);
+    F->setName(funcName + "_original");
+    L = Function::Create(ft,
+                         Function::ExternalLinkage,
+                         funcName,
+                         &M);
+  } else
+    L = Function::Create(ft,
+                         Function::ExternalLinkage,
+                         "_pocl_launcher_" + funcName,
+                         &M);
+
+  SmallVector<Value *, 8> arguments;
+  Function::arg_iterator ai = L->arg_begin();
+  for (unsigned i = 0, e = F->getArgumentList().size(); i != e; ++i) {
+    arguments.push_back(&*ai);
+    ++ai;
+  }
+
+  // Copy the function attributes to transfer noalias etc. from the
+  // original kernel which will be inlined into the launcher.
+  L->setAttributes(F->getAttributes());
+
   IRBuilder<> builder(BasicBlock::Create(M.getContext(), "", L));
 
-#ifdef LLVM_OLDER_THAN_3_7
-  ptr = builder.CreateStructGEP(ai,
-				TypeBuilder<PoclContext, true>::WORK_DIM);
-#else
-  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
-                                TypeBuilder<PoclContext, true>::WORK_DIM);
-#endif
-  gv = M.getGlobalVariable("_work_dim");
+  GlobalVariable *gv = M.getGlobalVariable("_work_dim");
   if (gv != NULL) {
-    v = builder.CreateLoad(builder.CreateConstGEP1_32(ptr, 0));
+    Value *ptr;
+#ifdef LLVM_OLDER_THAN_3_7
+    ptr =
+      builder.CreateStructGEP(ai, TypeBuilder<PoclContext, true>::WORK_DIM);
+#else
+    ptr =
+      builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
+                              TypeBuilder<PoclContext, true>::WORK_DIM);
+#endif
+    Value *v = builder.CreateLoad(builder.CreateConstGEP1_32(ptr, 0));
     builder.CreateStore(v, gv);
   }
 
-
   int size_t_width = 32;
-#ifdef LLVM_OLDER_THAN_3_7
-  if (M.getDataLayout()->getPointerSize(0) == 8)
-#else
-  if (M.getDataLayout().getPointerSize(0) == 8)
-#endif
+  if (currentPoclDevice->address_bits == 64)
     size_t_width = 64;
 
 #ifdef LLVM_OLDER_THAN_3_7
-  ptr = builder.CreateStructGEP(ai,
-				TypeBuilder<PoclContext, true>::GROUP_ID);
+  llvm::Argument *a = ai;
 #else
-  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
-                                TypeBuilder<PoclContext, true>::GROUP_ID);
+  llvm::Argument *a = &*ai;
 #endif
-  for (int i = 0; i < 3; ++i) {
-    snprintf(s, STRING_LENGTH, "_group_id_%c", 'x' + i);
-    gv = M.getGlobalVariable(s);
-    if (gv != NULL) {
-      if (size_t_width == 64)
-        {
-          v = builder.CreateLoad(builder.CreateConstGEP2_64(ptr, 0, i));
-        }
-      else
-        {
-#ifdef LLVM_OLDER_THAN_3_7
-          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr, 0, i));
-#else
-          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr->getType()->getPointerElementType(), ptr, 0, i));
-#endif
-        }
-      builder.CreateStore(v, gv);
-    }
-  }
 
-#ifdef LLVM_OLDER_THAN_3_7
-  ptr = builder.CreateStructGEP(ai,
-				TypeBuilder<PoclContext, true>::NUM_GROUPS);
-#else
-  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
-                                TypeBuilder<PoclContext, true>::NUM_GROUPS);
-#endif
-  for (int i = 0; i < 3; ++i) {
-    snprintf(s, STRING_LENGTH, "_num_groups_%c", 'x' + i);
-    gv = M.getGlobalVariable(s);
-    if (gv != NULL) {
-      if (size_t_width == 64)
-        {
-          v = builder.CreateLoad(builder.CreateConstGEP2_64(ptr, 0, i));
-        }
-      else 
-        {
-#ifdef LLVM_OLDER_THAN_3_7
-          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr, 0, i));
-#else
-          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr->getType()->getPointerElementType(), ptr, 0, i));
-#endif
-        }
-      builder.CreateStore(v, gv);
-    }
-  }
+  addGEPs(M, builder, a, size_t_width, TypeBuilder<PoclContext, true>::GROUP_ID,
+          "_group_id_%c");
 
-#ifdef LLVM_OLDER_THAN_3_7
-  ptr = builder.CreateStructGEP(ai,
-				TypeBuilder<PoclContext, true>::GLOBAL_OFFSET);
-#else
-  ptr = builder.CreateStructGEP(ai->getType()->getPointerElementType(), &*ai,
-                                TypeBuilder<PoclContext, true>::GLOBAL_OFFSET);
-#endif
-  for (int i = 0; i < 3; ++i) {
-    snprintf(s, STRING_LENGTH, "_global_offset_%c", 'x' + i);
-    gv = M.getGlobalVariable(s);
-    if (gv != NULL) {
-      if (size_t_width == 64)
-        {
-          v = builder.CreateLoad(builder.CreateConstGEP2_64(ptr, 0, i));
-        }
-      else
-        {
-#ifdef LLVM_OLDER_THAN_3_7
-          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr, 0, i));
-#else
-          v = builder.CreateLoad(builder.CreateConstGEP2_32(ptr->getType()->getPointerElementType(), ptr, 0, i));
-#endif
-        }
-      builder.CreateStore(v, gv);
-    }
-  }
+  if (WGDynamicLocalSize)
+    addGEPs(M, builder, a, size_t_width, TypeBuilder<PoclContext, true>::LOCAL_SIZE,
+            "_local_size_%c");
+
+  addGEPs(M, builder, a, size_t_width, TypeBuilder<PoclContext, true>::NUM_GROUPS,
+          "_num_groups_%c");
+
+  addGEPs(M, builder, a, size_t_width, TypeBuilder<PoclContext, true>::GLOBAL_OFFSET,
+          "_global_offset_%c");
 
   CallInst *c = builder.CreateCall(F, ArrayRef<Value*>(arguments));
   builder.CreateRetVoid();
 
+#ifdef LLVM_4_0
+  // At least with LLVM 4.0, the runtime of AddAliasScopeMetadata of
+  // llvm::InlineFunction explodes in case of kernels with restrict
+  // metadata and a lot of lifetime markers. The issue produces at
+  // least with EinsteinToolkit which has a lot of restrict kernel
+  // args). Remove them here before inlining to speed it up.
+  // TODO: Investigate the root cause.
+
+  std::set<CallInst*> Calls;
+
+  for (Function::iterator I = F->begin(), E = F->end(); I != E; ++I) {
+    for (BasicBlock::iterator BI = I->begin(), BE = I->end(); BI != BE; ++BI) {
+      Instruction *Instr = dyn_cast<Instruction>(BI);
+      if (!llvm::isa<CallInst>(Instr)) continue;
+      CallInst *CallInstr = dyn_cast<CallInst>(Instr);
+      if (CallInstr->getCalledFunction() != nullptr &&
+          (CallInstr->getCalledFunction()->getName() ==
+           "llvm.lifetime.end" ||
+           CallInstr->getCalledFunction()->getName() ==
+           "llvm.lifetime.start")) {
+        Calls.insert(CallInstr);
+      }
+    }
+  }
+
+  for (auto C : Calls) {
+    C->eraseFromParent();
+  }
+#endif
+
+
+  // TODO: Get rid of this step as inlining takes surprisingly long
+  // with large functions and the effect to total kernel runtime
+  // is meaningless.
   InlineFunctionInfo IFI;
   InlineFunction(c, IFI);
-  
+
   return L;
 }
 
@@ -534,7 +567,7 @@ createWorkgroup(Module &M, Function *F)
   }
 
   arguments.back() = &*(++ai);
-  
+
   builder.CreateCall(F, ArrayRef<Value*>(arguments));
   builder.CreateRetVoid();
 }
@@ -599,15 +632,20 @@ createWorkgroupFast(Module &M, Function *F)
     Value *value;
 
     if (!ii->hasByValAttr() || ((PointerType*)t)->getAddressSpace() == 1)
+#ifdef POCL_USE_FAKE_ADDR_SPACE_IDS
       value = builder.CreatePointerCast
-        (pointer, t->getPointerTo(POCL_ADDRESS_SPACE_GLOBAL));
+        (pointer, t->getPointerTo(POCL_FAKE_AS_GLOBAL));
+#else
+      value = builder.CreatePointerCast
+        (pointer, t->getPointerTo(currentPoclDevice->global_as_id));
+#endif
     else
       value = builder.CreatePointerCast(pointer, t->getPointerTo());
 
     if (!ii->hasByValAttr()) {
       value = builder.CreateLoad(value);
     }
-    
+
     arguments.push_back(value);
   }
 
@@ -627,6 +665,9 @@ Workgroup::isKernelToProcess(const Function &F)
 {
   const Module *m = F.getParent();
 
+  if (F.getMetadata("kernel_arg_access_qual"))
+    return true;
+
   NamedMDNode *kernels = m->getNamedMetadata("opencl.kernels");
   if (kernels == NULL) {
     if (KernelName == "")
@@ -635,15 +676,15 @@ Workgroup::isKernelToProcess(const Function &F)
       return true;
 
     return false;
-  }  
+  }
 
   for (unsigned i = 0, e = kernels->getNumOperands(); i != e; ++i) {
     if (kernels->getOperand(i)->getOperand(0) == NULL)
       continue; // globaldce might have removed uncalled kernels
-    Function *k = 
+    Function *k =
       cast<Function>(
         dyn_cast<ValueAsMetadata>(kernels->getOperand(i)->getOperand(0))
-          ->getValue());
+        ->getValue());
     if (&F == k)
       return true;
   }
