@@ -72,8 +72,12 @@ char WorkitemLoops::ID = 0;
 void
 WorkitemLoops::getAnalysisUsage(AnalysisUsage &AU) const
 {
-
+#ifdef LLVM_OLDER_THAN_3_9
   AU.addRequired<PostDominatorTree>();
+#else
+  AU.addRequired<PostDominatorTreeWrapperPass>();
+#endif
+
 #ifdef LLVM_OLDER_THAN_3_7
   AU.addRequired<LoopInfo>();
 #else
@@ -106,7 +110,12 @@ WorkitemLoops::runOnFunction(Function &F)
 #else
   LI = &getAnalysis<LoopInfoWrapperPass>();
 #endif
+
+#ifdef LLVM_OLDER_THAN_3_9
   PDT = &getAnalysis<PostDominatorTree>();
+#else
+  PDT = &getAnalysis<PostDominatorTreeWrapperPass>();
+#endif
 
   tempInstructionIndex = 0;
 
@@ -140,9 +149,9 @@ WorkitemLoops::runOnFunction(Function &F)
 std::pair<llvm::BasicBlock *, llvm::BasicBlock *>
 WorkitemLoops::CreateLoopAround
 (ParallelRegion &region,
- llvm::BasicBlock *entryBB, llvm::BasicBlock *exitBB, 
+ llvm::BasicBlock *entryBB, llvm::BasicBlock *exitBB,
  bool peeledFirst, llvm::Value *localIdVar, size_t LocalSizeForDim,
- bool addIncBlock) 
+ bool addIncBlock, llvm::Value *DynamicLocalSize)
 {
   assert (localIdVar != NULL);
 
@@ -263,13 +272,19 @@ WorkitemLoops::CreateLoopAround
     }
 
   builder.SetInsertPoint(forCondBB);
-  llvm::Value *cmpResult = 
-    builder.CreateICmpULT
-    (builder.CreateLoad(localIdVar),
-     (ConstantInt::get
-      (IntegerType::get(C, size_t_width), 
-       LocalSizeForDim)));
-      
+
+  llvm::Value *cmpResult;
+  if (!WGDynamicLocalSize)
+    cmpResult = builder.CreateICmpULT(
+                  builder.CreateLoad(localIdVar),
+                    ConstantInt::get(
+                      IntegerType::get(C, size_t_width),
+                      LocalSizeForDim));
+  else
+    cmpResult = builder.CreateICmpULT(
+                  builder.CreateLoad(localIdVar),
+                    builder.CreateLoad(DynamicLocalSize));
+  
   Instruction *loopBranch =
       builder.CreateCondBr(cmpResult, loopBodyEntryBB, loopEndBB);
 
@@ -323,6 +338,9 @@ bool
 WorkitemLoops::ProcessFunction(Function &F)
 {
   Kernel *K = cast<Kernel> (&F);
+  
+  llvm::Module *M = K->getParent();
+
   Initialize(K);
   unsigned workItemCount = WGLocalSizeX*WGLocalSizeY*WGLocalSizeZ;
 
@@ -498,20 +516,53 @@ WorkitemLoops::ProcessFunction(Function &F)
         }
       }
 
-    if (WGLocalSizeX > 1)
-      l = CreateLoopAround(
-        *original, l.first, l.second, peelFirst, 
-        localIdX, WGLocalSizeX, !unrolled);
+    if (WGDynamicLocalSize) {
+      GlobalVariable *gv;
+      gv = M->getGlobalVariable("_local_size_x");
+      auto *SizeT_Ty = Type::getIntNTy(M->getContext(), size_t_width);
+      if (gv == NULL) 
+        gv = new GlobalVariable(*M, SizeT_Ty, true, GlobalValue::CommonLinkage,
+                                NULL, "_local_size_x", NULL,
+                                GlobalValue::ThreadLocalMode::NotThreadLocal,
+                                0, true);
 
-    if (WGLocalSizeY > 1)
-      l = CreateLoopAround(
-        *original, l.first, l.second, 
-        false, localIdY, WGLocalSizeY);
+      l = CreateLoopAround(*original, l.first, l.second, peelFirst,
+                           localIdX, WGLocalSizeX, !unrolled, gv);
 
-    if (WGLocalSizeZ > 1)
-      l = CreateLoopAround(
-          *original, l.first, l.second, 
-          false, localIdZ, WGLocalSizeZ);
+      gv = M->getGlobalVariable("_local_size_y");
+      if (gv == NULL) 
+        gv = new GlobalVariable(*M, SizeT_Ty, false, GlobalValue::CommonLinkage,
+                                NULL, "_local_size_y");
+
+      l = CreateLoopAround(*original, l.first, l.second,
+                           false, localIdY, WGLocalSizeY, !unrolled, gv);
+
+      gv = M->getGlobalVariable("_local_size_z");
+      if (gv == NULL) 
+        gv = new GlobalVariable(*M, SizeT_Ty, true, GlobalValue::CommonLinkage,
+                                NULL, "_local_size_z", NULL,
+                                GlobalValue::ThreadLocalMode::NotThreadLocal,
+                                0, true);
+
+      l = CreateLoopAround(*original, l.first, l.second,
+                           false, localIdZ, WGLocalSizeZ, !unrolled, gv);
+
+    } else {
+      if (WGLocalSizeX > 1) {
+          l = CreateLoopAround(*original, l.first, l.second, peelFirst,
+                               localIdX, WGLocalSizeX, !unrolled);
+        }
+
+      if (WGLocalSizeY > 1) {
+          l = CreateLoopAround(*original, l.first, l.second, false,
+                               localIdY, WGLocalSizeY);
+        }
+
+      if (WGLocalSizeZ > 1) {
+          l = CreateLoopAround(*original, l.first, l.second, false,
+                               localIdZ, WGLocalSizeZ);
+        }
+    }
 
     /* Loop edges coming from another region mean B-loops which means 
        we have to fix the loop edge to jump to the beginning of the wi-loop 
@@ -548,7 +599,9 @@ WorkitemLoops::ProcessFunction(Function &F)
        localIdXFirstVar);       
   }
 
-  K->addLocalSizeInitCode(WGLocalSizeX, WGLocalSizeY, WGLocalSizeZ);
+  if (!WGDynamicLocalSize)
+    K->addLocalSizeInitCode(WGLocalSizeX, WGLocalSizeY, WGLocalSizeZ);
+
   ParallelRegion::insertLocalIdInit(&F.getEntryBlock(), 0, 0, 0);
 
 #if 0
@@ -636,6 +689,46 @@ WorkitemLoops::FixMultiRegionVariables(ParallelRegion *region)
     }
 }
 
+llvm::Value *
+WorkitemLoops::GetLinearWiIndex(llvm::IRBuilder<> &builder, llvm::Module *M,
+                               ParallelRegion *region)
+{
+  auto *SizeTType = Type::getIntNTy(M->getContext(), size_t_width);
+  GlobalVariable *LocalSizeXPtr =
+    cast<GlobalVariable>(M->getOrInsertGlobal("_local_size_x", SizeTType));
+  GlobalVariable *LocalSizeYPtr =
+    cast<GlobalVariable>(M->getOrInsertGlobal("_local_size_y", SizeTType));
+
+  assert(LocalSizeXPtr != NULL && LocalSizeYPtr != NULL);
+
+  LoadInst* LoadX = builder.CreateLoad(LocalSizeXPtr, "ls_x");
+  LoadInst* LoadY = builder.CreateLoad(LocalSizeYPtr, "ls_y");
+
+  /* Form linear index from xyz coordinates:
+       local_size_x * local_size_y * local_id_z  (z dimension)
+     + local_size_x * local_id_y                 (y dimension)
+     + local_id_x                                (x dimension)
+  */
+  Value* LocalSizeXTimesY =
+    builder.CreateBinOp(Instruction::Mul, LoadX, LoadY, "ls_xy");
+
+  Value* ZPart =
+    builder.CreateBinOp(Instruction::Mul, LocalSizeXTimesY,
+                        region->LocalIDZLoad(),
+                        "tmp");
+
+  Value* YPart =
+    builder.CreateBinOp(Instruction::Mul, LoadX, region->LocalIDYLoad(),
+                        "ls_x_y");
+
+  Value* ZYSum =
+    builder.CreateBinOp(Instruction::Add, ZPart, YPart,
+                        "zy_sum");
+
+  return builder.CreateBinOp(Instruction::Add, ZYSum, region->LocalIDXLoad(),
+                             "linear_xyz_idx");
+}
+
 llvm::Instruction *
 WorkitemLoops::AddContextSave
 (llvm::Instruction *instruction, llvm::Instruction *alloca)
@@ -661,19 +754,28 @@ WorkitemLoops::AddContextSave
 
   IRBuilder<> builder(&*definition);
   std::vector<llvm::Value *> gepArgs;
-  gepArgs.push_back(ConstantInt::get(IntegerType::get(instruction->getContext(), size_t_width), 0));
-
+  
   /* Reuse the id loads earlier in the region, if possible, to
      avoid messy output with lots of redundant loads. */
   ParallelRegion *region = RegionOfBlock(instruction->getParent());
   assert ("Adding context save outside any region produces illegal code." && 
           region != NULL);
 
-  gepArgs.push_back(region->LocalIDZLoad());
-  gepArgs.push_back(region->LocalIDYLoad());
-  gepArgs.push_back(region->LocalIDXLoad());
+  if (WGDynamicLocalSize)
+    {
+      Module *M = alloca->getParent()->getParent()->getParent();
+      gepArgs.push_back(GetLinearWiIndex(builder, M, region));
+    }
+  else
+    {
+      gepArgs.push_back(ConstantInt::get(IntegerType::get(instruction->getContext(), size_t_width), 0));
+      gepArgs.push_back(region->LocalIDZLoad());
+      gepArgs.push_back(region->LocalIDYLoad());
+      gepArgs.push_back(region->LocalIDXLoad());
+    }
 
-  return builder.CreateStore(instruction, builder.CreateGEP(alloca, gepArgs));
+  return builder.CreateStore(instruction, builder.CreateGEP(alloca,
+                                                            gepArgs));
 }
 
 llvm::Instruction *
@@ -698,9 +800,7 @@ WorkitemLoops::AddContextRestore
       assert (false && "Unknown context restore location!");
     }
 
-  
   std::vector<llvm::Value *> gepArgs;
-  gepArgs.push_back(ConstantInt::get(IntegerType::get(val->getContext(), size_t_width), 0));
 
   /* Reuse the id loads earlier in the region, if possible, to
      avoid messy output with lots of redundant loads. */
@@ -708,19 +808,28 @@ WorkitemLoops::AddContextRestore
   assert ("Adding context save outside any region produces illegal code." && 
           region != NULL);
 
-  gepArgs.push_back(region->LocalIDZLoad());
-  gepArgs.push_back(region->LocalIDYLoad());
-  gepArgs.push_back(region->LocalIDXLoad());
+  if (WGDynamicLocalSize)
+    {
+      Module *M = alloca->getParent()->getParent()->getParent();
+      gepArgs.push_back(GetLinearWiIndex(builder, M, region));
+    }
+  else
+    {
+      gepArgs.push_back(ConstantInt::get(IntegerType::get(val->getContext(),
+                                                          size_t_width), 0));
+      gepArgs.push_back(region->LocalIDZLoad());
+      gepArgs.push_back(region->LocalIDYLoad());
+      gepArgs.push_back(region->LocalIDXLoad());
+    }
 
-
-  llvm::Instruction *gep = 
+  llvm::Instruction *gep =
     dyn_cast<Instruction>(builder.CreateGEP(alloca, gepArgs));
   if (isAlloca) {
     /* In case the context saved instruction was an alloca, we created a
        context array with pointed-to elements, and now want to return a pointer 
        to the elements to emulate the original alloca. */
     return gep;
-  }           
+  }
   return builder.CreateLoad(gep);
 }
 
@@ -781,25 +890,52 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction)
       elementType = instruction->getType();
     }
 
-  /* 3D context array. */
-  llvm::Type *contextArrayType = 
-    ArrayType::get(
+  llvm::AllocaInst *Alloca;
+  Module* M = instruction->getParent()->getParent()->getParent();
+  if (WGDynamicLocalSize)
+    {
+      char GlobalName[32];
+      GlobalVariable* LocalSize;
+      LoadInst* LocalSizeLoad[3];
+      auto *SizeT_Ty = Type::getIntNTy(M->getContext(), size_t_width);
+      for (int i = 0; i < 3; ++i) {
+        snprintf(GlobalName, 32, "_local_size_%c", 'x' + i);
+        LocalSize =
+          cast<GlobalVariable>(M->getOrInsertGlobal(GlobalName, SizeT_Ty));
+        LocalSizeLoad[i] = builder.CreateLoad(LocalSize);
+      }
+
+      Value* LocalXTimesY =
+        builder.CreateBinOp(Instruction::Mul, LocalSizeLoad[0],
+                            LocalSizeLoad[1], "tmp");
+      Value* NumberOfWorkItems =
+        builder.CreateBinOp(Instruction::Mul, LocalXTimesY,
+                            LocalSizeLoad[2], "num_wi");
+
+      Alloca = builder.CreateAlloca(elementType, NumberOfWorkItems, varName);
+    }
+  else
+    {
+      /* 3D context array. */
+      llvm::Type *contextArrayType =
         ArrayType::get(
+          ArrayType::get(
             ArrayType::get(
-                elementType, WGLocalSizeX), 
+                           elementType, WGLocalSizeX),
             WGLocalSizeY), WGLocalSizeZ);
 
-  /* Allocate the context data array for the variable. */
-  llvm::AllocaInst *alloca = 
-    builder.CreateAlloca(contextArrayType, 0, varName);
+      /* Allocate the context data array for the variable. */
+      Alloca = builder.CreateAlloca(contextArrayType, 0, varName);
+    }
+
   /* Align the context arrays to stack to enable wide vectors
      accesses to them. Also, LLVM 3.3 seems to produce illegal
      code at least with Core i5 when aligned only at the element
      size. */
-  alloca->setAlignment(CONTEXT_ARRAY_ALIGN);
+  Alloca->setAlignment(CONTEXT_ARRAY_ALIGN);
 
-  contextArrays[varName] = alloca;
-  return alloca;
+  contextArrays[varName] = Alloca;
+  return Alloca;
 }
 
 
